﻿"""
/***************************************************************************
Name                  : Mask 
Description          : Aide à la création de masque
Date                 : Feb/12 
copyright            : (C) 2011 by AEAG
email                : xavier.culos@eau-adour-garonne.fr 
todo: 

 add help file
 let's user change user defined parameters. 
 use a qml file to set masqk style. default qml inside plugin ? 
   Parameter 1: choose memory output or shp output + directory (default HOME directory)
   Parameter 2 : choose a qml file. Qml should be parsed to check if qymbology apply to polygon features

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
import os
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from qgis.core import *

# Initialize Qt resources from file resources.py
import resources

try:
    _fromUtf8 = QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class aeag_mask: 

  def __init__(self, iface):
    # Save reference to the QGIS interface
    self.iface = iface
    self.toolBar = None
    self.act_aeag_mask = None
    self.act_aeag_toolbar_help = None
    self.canvas = self.iface.mapCanvas()
    self.active = False
    self.read()
    
  def store(self):
    s = QSettings() 

  def read(self):
    s = QSettings()

  def initGui(self):  
    self.toolBar = self.iface.pluginToolBar()

    self.act_aeag_mask = QAction(QIcon(":plugins/mask/aeag_mask.png"), _fromUtf8("Create mask"), self.iface.mainWindow())
    self.act_aeag_mask.setCheckable(True)
    self.toolBar.addAction(self.act_aeag_mask)
    
    self.iface.addPluginToMenu("&Mask", self.act_aeag_mask)    

    # Add actions to the toolbar
    QObject.connect(self.act_aeag_mask, SIGNAL("triggered()"), self.do_aeag_mask)
    
    #add a link to helpfile for toolbar & menu aeag , if not exist
    

  def unload(self):
    self.toolBar.removeAction(self.act_aeag_mask)
    self.iface.removePluginMenu("&Mask", self.act_aeag_mask)
    self.store()

  # run method that performs all the real work
  def do_aeag_mask(self): 
    self.active = True
    layermap = QgsMapLayerRegistry.instance().mapLayers()
 
    g = None
    theLayer = None
    layerName = "Mask"
    for name, layer in layermap.iteritems():
        if layer.type() == QgsMapLayer.VectorLayer:
            if layer.isValid():
                theLayer = layer
                features = layer.selectedFeatures()
                for feature in features:
                    if feature.geometry().type() == QGis.Polygon:
                        if g == None:
                            g = feature.geometry()
                            f = feature
                            try:
                                layerName = "Mask " + feature.attributeMap()[layer.dataProvider().fieldNameIndex(layer.displayField())].toString()
                            except:
                                layerName = "Mask"
                        else:
                            g = g.combine(feature.geometry())
                            layerName = "Mask"
                            
            layer.removeSelection()  # remove user selection after use (clean screen from yellow display)
            
    if g != None:
        rect = self.canvas.extent()
        rect.scale(2)

        mask = QgsGeometry.fromRect(rect)

        toCrs = theLayer.crs();
        fromCrs = self.canvas.mapRenderer().destinationCrs()
        xform = QgsCoordinateTransform(fromCrs, toCrs)
        
        if self.canvas.hasCrsTransformEnabled():
            mask.transform(xform)

        mask.makeDifference(g)

        maskLayer = QgsVectorLayer("MultiPolygon", layerName, "memory")
        maskLayer.setCrs(toCrs)

        QgsMapLayerRegistry.instance().addMapLayer(maskLayer)

        pr = maskLayer.dataProvider()
        fet = QgsFeature()
        fet.setGeometry(mask)
        pr.addFeatures([ fet ])
        maskLayer.updateExtents()        

        # style
        maskLayer.setRenderer(QgsSingleSymbolRenderer(QGis.Polygon))
        if maskLayer.isUsingRendererV2():
            # new symbology
            rendererV2 = maskLayer.rendererV2()
            for s in rendererV2.symbols():
                s.setAlpha(0.75)
                s.setColor(QColor(168, 168, 168))
                
                if isinstance(s, QgsLineSymbolV2):
                    s.setWidth(0)
        else:
            # old symbology 
            maskLayer.setTransparency(5)
            
        # Refresh legend
        self.iface.legendInterface().refreshLayerSymbology( maskLayer ) 
        QgsMapLayerRegistry.instance().clearAllLayerCaches () #clean cache to allow mask layer to appear on refresh
        self.canvas.refresh()
       
    else:
        QMessageBox.information(None, "Attention", _fromUtf8("A selection of polygons is needed..."))

    self.act_aeag_mask.setChecked(False)
    self.active = False

  def deactivate(self):
        self.act_aeag_mask.setChecked(False)
        self.active = False
